<?php
/*+**********************************************************************************
 * The contents of this file are subject to the CRM Public License Version 1.0
 * ("License"); You may not use this file except in compliance with the License
 * The Original Code is:   CRM Open Source
 * The Initial Developer of the Original Code is .
 * Portions created by  are Copyright (C) .
 * All Rights Reserved.
 ************************************************************************************/

include_once dirname(__FILE__) . '/../ISMSProvider.php';
include_once 'vtlib/Vtiger/Net/Client.php';
include_once 'include/Zend/Json.php';

class SMSNotifier_eBulksms_Provider implements SMSNotifier_ISMSProvider_Model {

	private $username;
	private $password;
	private $parameters = array();

	const SERVICE_URI = 'https://api.ebulksms.com';
	private static $REQUIRED_PARAMETERS = array('sender', 'flash');


	/**
	 * Function to get provider name
	 * @return <String> provider name
	 */
	public function getName() {
		return 'eBulksms';
	}

	/**
	 * Function to get required parameters other than (username, password)
	 * @return <array> required parameters list
	 */
	public function getRequiredParams() {
		return self::$REQUIRED_PARAMETERS;
	}

	/**
	 * Function to get service URL to use for a given type
	 * @param <String> $type like SEND, PING, QUERY
	 */
	public function getServiceURL($type = false) {
		if($type) {
			switch(strtoupper($type)) {
				case self::SERVICE_AUTH: return  self::SERVICE_URI;
                                case self::SERVICE_SEND: return  self::SERVICE_URI . '/sendsms.json';
				case self::SERVICE_QUERY: return self::SERVICE_URI . '/getdlr.json';
			}
		}
		return false;
	}

	/**
	 * Function to set authentication parameters
	 * @param <String> $username
	 * @param <String> $password
	 */
	public function setAuthParameters($username, $password) {
		$this->username = $username;
		$this->password = $password;
	}

	/**
	 * Function to set non-auth parameter.
	 * @param <String> $key
	 * @param <String> $value
	 */
	public function setParameter($key, $value) {
		$this->parameters[$key] = $value;
	}

	/**
	 * Function to get parameter value
	 * @param <String> $key
	 * @param <String> $defaultValue
	 * @return <String> value/$default value
	 */
	public function getParameter($key, $defaultValue = false) {
		if(isset($this->parameters[$key])) {
			return $this->parameters[$key];
		}
		return $defaultValue;
	}

	/**
	 * Function to prepare parameters
	 * @return <Array> parameters
	 */
	protected function prepareParameters() {
		$params = array('username' => $this->username, 'apikey' => $this->password);
		foreach (self::$REQUIRED_PARAMETERS as $key) {
			$params[$key] = $this->getParameter($key);
		}
		return $params;
	}

        /**
	 * Function to handle SMS Send operation
	 * @param <String> $message
	 * @param <Mixed> $toNumbers One or Array of numbers
	 */
	public function send($message, $toNumbers) {
		if(!is_array($toNumbers)) {
			$toNumbers = array($toNumbers);
		}
                $gsm = array();
                foreach ($toNumbers as $recipient) {
                    $mobilenumber = trim($recipient);
                    $generated_id = uniqid('int_', false);
                    $generated_id = substr($generated_id, 0, 30);
                    $gsm['gsm'][] = array('msidn' => $mobilenumber, 'msgid' => $generated_id);
                }

		$params = $this->prepareParameters();
		$message = array(
                    'sender' => $params['sender'],
                    'messagetext' => $message,
                    'flash' => $params['flash'],
                );

                $request = array('SMS' => array(
                        'auth' => array(
                            'username' => $params['username'],
                            'apikey' => $params['apikey']
                        ),
                        'message' => $message,
                        'recipients' => $gsm
                ));
                $json_data = Zend_Json::encode($request);

		$serviceURL = $this->getServiceURL(self::SERVICE_SEND);
		$httpClient = new Vtiger_Net_Client($serviceURL);
                $httpClient->setHeaders(array('Content-Type' => 'application/json'));
		$response = $httpClient->doPost($json_data);
		$jsonResponse = Zend_Json::decode($response);
                
		$results = array();
                if($jsonResponse->response->status == 'SUCCESS'){
                    foreach($gsm['gsm'] as $recipient){
                        $results[] = array('id' => $recipient['msgid'], 'to' => $recipient['msidn'], 'statusmessage' => self::MSG_STATUS_PROCESSING);
                    }
                }
                else{
                    foreach($gsm['gsm'] as $recipient){
                        $results[] = array('error' => true, 'to' => $recipient['msidn'], 'statusmessage' => $jsonResponse->response->status );
                    }
                }
                return $results;
	}


	/**
	 * Function to get query for status using messgae id
	 * @param <Number> $messageId
	 */
	public function query($messageId) {
		$params = $this->prepareParameters();
		$params['uniqueid'] = $messageId;

		$serviceURL = $this->getServiceURL(self::SERVICE_QUERY);
		$httpClient = new Vtiger_Net_Client($serviceURL);
		$response = $httpClient->doGet($params);
                
                $result = array();
                if($response == 'AUTH_FAILURE'){
                    $result = array('error' => true, 'needlookup' => 0, 'statusmessage' => 'AUTH_FAILURE');
                }
                else{
                    $jsonResponse = Zend_Json::decode($response);
                    $result = array('id' => $jsonResponse->dlr[0]->id, 'to' => $jsonResponse->dlr[0]->mobilenumber, 'statusmessage' => $jsonResponse->dlr[0]->status);
                }
		return $result;
	}
}
?>